
/* RCS Information:
 *
 * $Id$
 * $Revision$
 * $Date$
 * $Author$
 * $Locker$
 * 
 */

/******************************************************************************
 *
 *  Copyright (C) 2002, Bill Pierce
 *  This file is part of the hbaapiutils library.
 *
 *  The hbaapiutils library is free software; you can 
 *  redistribute it and/or modify it under the terms of the 
 *  GNU Library General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The hbaapiutils library is distributed in the hope that 
 *  it will be useful, but WITHOUT ANY WARRANTY; without even 
 *  the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 *  PARTICULAR PURPOSE.  See the GNU Library General Public 
 *  License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with the timeout action libary; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Bill Pierce can be contacted at:
 *  7862 NE 148th St.
 *  Kenmore, WA 98028
 *
 * Name -- hbaapiutils.c 
 *
 * Purpose -- These routines assist in the 
 *            development of applications that use the  
 *            functionality provided by the SNIA HBA API. 
 *
 * Remarks --  
 *
 * Known bugs --  
 *
 * Supported Platforms -- Windows NT, 2000, UNIX flavors. Should be portable.
 *
 * Applicable Invariants -- 
 * 
 * Language Version --  Standard C  
 *
 * Author --  Bill Pierce 
 *
 * Origination Date -- 8-21-01
 * 
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include "hbaapi.h"

/* Some constants used in WWN conversion routines below. */
#define WWN_STRING_LENGTH 23
#define WWN_BYTE_DELIMITER_STRING ":"

/*****************************************************************************
 *
 * Library of utility functions to make working with the HBA API easier.
 *
 *****************************************************************************/

/* Convert a LUN from an integer value into an HBA_UINT64.
 * Read the value from lun64 and write it to p_lun_as_int.
 * This implementation assumes the lun number is embedded 
 * in the second byte of the 8 byte FCP lun according to the
 * SAM-2 specification section 4.8.
 * Return 1 if the operation succeeds, 0 if not.
 */
int Lun64ToInt(HBA_UINT64 lun64, int* p_lun_as_int) {
  int return_code = 1;
  unsigned char * p_byte;
  /* Check precondition */
  if (NULL == p_lun_as_int) {
     return 0;
  }
  p_byte = (unsigned char *) &lun64;
  *p_lun_as_int = (int) p_byte[1];
  return return_code;
}

/* Convert a LUN from an HBA_UINT64 value into an integer.
 * Read the value from lun_as_int and write to p_lun64.
 * This implementation assumes the lun number is embedded 
 * in the second byte of the 8 byte FCP lun according to the
 * SAM-2 specification section 4.8.
 * Return 1 if the operation succeeds, 0 if not.
 */ 
int IntToLun64(int lun_as_int, HBA_UINT64* p_lun64) {
  int return_code = 1;
  unsigned char * p_byte;
  /* Check precondition */
  if (NULL == p_lun64) {
     return 0;
  }
  /* Zero the bytes. */
  memset(p_lun64, 0, sizeof(HBA_UINT64));
  p_byte = (unsigned char *) p_lun64;
  p_byte[1] = (unsigned char) lun_as_int;
  return return_code;
}

/** Take a string and determine if it is a valid, uniform format,
 *  World Wide Name. Currently, this implementation looks for 
 *  a string the correct size containing 2 digit hex numbers 
 *  delimited by colons.  Return 1 if true, 0 if false.
 */
int IsAValidWWN(const char *pWWNAsString) {

   char candidate[WWN_STRING_LENGTH+1];
   int byte_token_count;
   char *byte_token;
   unsigned short byteval;
   int parse_success;
   
   /* First make sure we are the right length. */
   if (WWN_STRING_LENGTH != strlen(pWWNAsString)) {
     return 0;
   } else {
     memset(candidate, '\0', WWN_STRING_LENGTH+1);
     strcpy(candidate, pWWNAsString);
   }

   /* Now go through token by token. */
   byte_token_count = 0;
   byte_token = strtok(candidate, WWN_BYTE_DELIMITER_STRING);
   while (NULL != byte_token) {
       byte_token_count++;
       parse_success = sscanf(byte_token, "%2hX", &byteval);
       if (1 == parse_success) {
          if (byteval > 255) {
             return 0;
          } 
       } else {
          return 0;
       } 
       byte_token = strtok(NULL, WWN_BYTE_DELIMITER_STRING);
   }

   /* Were there the right number of bytes? */
   if (sizeof(HBA_WWN) != byte_token_count) {
      return 0;
   }
     
   /* If we got here, we must have passed the tests. */
   return 1;
} /* End of IsAValidWWN. */
  
/***************************************************************************
** Take a string containing a uniform format wwn and 
** the worldwidename as as an array of 8 unsigned bytes (char)
** and convert it to a uniform string format. Return 1 on success 0 on error.
****************************************************************************/
int StrToWWN(const char *pWWNAsString, HBA_WWN *pWWN)
{   
    unsigned short read_array[8];
    int scanf_return = 0;
    int success;
    int i;

    /* Initialize our state. */
    success = 0;
    for (i=0;i<8;i++) {
       read_array[i] = 0;
    }

    /* Verify the format of the string. */
    if (!IsAValidWWN(pWWNAsString)) {
       return 0;
    }

    /* Read the string into an array of numbers. */
    scanf_return =
    sscanf(pWWNAsString, "%2hX:%2hX:%2hX:%2hX:%2hX:%2hX:%2hX:%2hX",
            &read_array[0],
            &read_array[1],
            &read_array[2],
            &read_array[3],
            &read_array[4],
            &read_array[5],
            &read_array[6],
            &read_array[7]
           );

    /* Make sure we got all that we needed. */
    if (8 != scanf_return) {
       for (i=0;i<8;i++) {
          read_array[i] = 0;
       }
       success = 0;
    } else {
       success = 1;
    }

    /* Further validation goes here. */

    /* Copy a result into the wwn structure. */
    for (i=0;i<8;i++) {
       pWWN->wwn[i] = (HBA_UINT8) read_array[i];
    }

    return success;
}

/***************************************************************************
** Take the worldwidename as as an array of 8 unsigned bytes (char)
** and convert it to a string format.
****************************************************************************/
void WWNToStr(const HBA_WWN *pWWN, char *pWWNAsString)
{   
    unsigned char array[8];
    memcpy(array, pWWN, 8);
    sprintf(pWWNAsString, "%02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X",
            (int)array[0],
            (int)array[1],
            (int)array[2],
            (int)array[3],
            (int)array[4],
            (int)array[5],
            (int)array[6],
            (int)array[7]
           );
}

/****************************************************************************
** Utility function that returns a string representing the return status.
****************************************************************************/
char * StatusToStr(HBA_STATUS Status)
{
    char *pRet = "UNRECOGNIZED";


    switch (Status)
    {
        case HBA_STATUS_OK:
            pRet = "SUCCESS";
        break;

        case HBA_STATUS_ERROR:
            pRet = "ERROR";
        break;

        case HBA_STATUS_ERROR_NOT_SUPPORTED:
            pRet = "NOT SUPPORTED";
        break;

        case HBA_STATUS_ERROR_INVALID_HANDLE:
            pRet = "INVALID HANDLE";
        break;

        case HBA_STATUS_ERROR_ARG:
            pRet = "BAD ARG";
        break;

        case HBA_STATUS_ERROR_ILLEGAL_WWN:
            pRet = "ILLEGAL WWN";
        break;

        case HBA_STATUS_ERROR_ILLEGAL_INDEX:
            pRet = "ILLEGAL INDEX";
        break;

        case HBA_STATUS_ERROR_MORE_DATA:
            pRet = "BUFFER TOO SMALL";
        break;

        case HBA_STATUS_ERROR_STALE_DATA:
            pRet = "STALE DATA, REFRESH REQUIRED";
        break;

        case HBA_STATUS_SCSI_CHECK_CONDITION:
            pRet = "SCSI SENSE INFO EXISTS";
        break;

	default:
	break;

    }

    return (pRet);

}  /* End of function "StatusToStr" */

/*****************************************************************************
 *
 *   $Log: $
 *
 *****************************************************************************/
